
*************************************************************
DEFINE CLASS MCI AS Custom
*************************************************************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1995
***   Contact: (503) 386-2087  / 76427,2363@compuserve.com
***  Modified: 06/19/95
***
***  Function: Plays MCI device files
***    Assume: Proper MCI Device is installed
*************************************************************
*** Custom Properties
lLoadFoxTools=.F.
cMCIObject=""
cFileName=""
cFileType=""                  && Default File Type
cLastError=""
lClosed=.F.
nTrackLength=0

*** Visual MCI Devices - Original Device Window Size
nWidth=0
nHeight=0

************************************************************************
* MCI :: Init
******************
***      Pass: lcFileName  -  Name of MCI file to play
************************************************************************
PROCEDURE Init
LPARAMETER lcFileName

DECLARE INTEGER mciSendString ;
   IN WinMM.DLL ;
   STRING cMCIString,;
   STRING @cRetString,;
   INTEGER nRetLength,;
   INTEGER hInstance

THIS.cMCIObject=SYS(3)

ENDPROC
* Init

************************************************************************
* MCI :: Destroy
******************
***  Function: Shutdown code closes MCI device
************************************************************************
PROCEDURE Destroy
IF !THIS.lClosed
   THIS.close
ENDIF

ENDPROC
* Destroy


************************************************************************
* MCI :: Open
******************
***  Function: Loads a file - Virtual Method
***      Pass: lcFileName   -  Name of file to open (if any)
***            lcFileType   -  File extension
***            lcOptions    -  Optional MCI Command strings
***    Return: .T. or .F.
************************************************************************
PROCEDURE OPEN
LPARAMETERS lcFileName,lcFileType,lcOptions
LOCAL lnTop,lnLeft,lnHeight,lnWidth

lcFileType=IIF(TYPE("lcFileType")="C",lcFileType,"*")
lcFileName=IIF(TYPE("lcFileName")="C",lcFileName,GETFILE(lcFileType,"Open File"))
lcOptions=IIF(TYPE("lcOptions")="C",lcOptions,"")

IF !FILE(lcFilename)
   lcFileName=GETFILE(lcFileType,"Select "+lcFileType+" file:","Select",0)
   IF !FILE(lcFileName)
      lcFilename=""
      THIS.cFileName=""
      RETURN .F.
   ENDIF
ENDIF

THIS.cFileName=FULLPATH(lcFilename)

IF THIS.mciExec("open "+THIS.cFileName+" ALIAS "+THIS.cMCIObject+" "+lcOptions)#"*ERROR*"
   THIS.lClosed=.F.
   THIS.nTrackLength=VAL ( THIS.MCIExec("Status "+THIS.cMCIObject+" length") )

   *** Save original Window Size
   STORE 0 TO lnWidth,lnHeight
   THIS.getsize(@lnWidth,@lnHeight)

   THIS.nWidth=lnWidth
   THIS.nHeight=lnHeight

   *** Force all devices to default to milliseconds
   THIS.SetTimeFormat("MILLISECONDS")

   RETURN .T.
ENDIF

RETURN .F.
* Open


************************************************************************
* MCI :: OpenForRecording
**************************
***  Function: As Open, but for recording devices, which need
***            to record to a new device.
***    Assume:
***      Pass: lcFilename   -   Name to record to
***            lcFileType   -   WAV AVI etc.
***    Return: .t. or .f.
******************************************************* *****************
PROCEDURE OpenForRecording
LPARAMETERS lcFileName,lcFileType,lcOptions
LOCAL lnTop,lnLeft,lnHeight,lnWidth

lcFileType=IIF(TYPE("lcFileType")="C",lcFileType,"*")
lcFileName=IIF(TYPE("lcFileName")="C",lcFileName,GETFILE(lcFileType,"Open File"))
lcOptions=IIF(TYPE("lcOptions")="C",lcOptions,"")

THIS.cFileName=FULLPATH(lcFilename)

*** Make sure we're starting fresh
ERASE(THIS.cFilename)

IF THIS.mciExec("open new ALIAS "+THIS.cMCIObject+" type "+lcFileType+" "+lcOptions)#"*ERROR*"
   THIS.lClosed=.F.

   *** Save original Window Size
   STORE 0 TO lnWidth,lnHeight
   THIS.getsize(@lnWidth,@lnHeight)

   THIS.nWidth=lnWidth
   THIS.nHeight=lnHeight

   *** Force all devices to default to milliseconds
   THIS.SetTimeFormat("MILLISECONDS")

   RETURN .T.
ENDIF

RETURN .F.
* OpenRec


************************************************************************
* MCI :: Close
******************
***  Function: Closes MCI device
***    Return: .T. or .F.
************************************************************************
PROCEDURE CLOSE
IF THIS.mciExec("Close "+THIS.cMCIObject)="*ERROR*"
   RETURN .F.
ENDIF

THIS.lClosed=.T.
RETURN .T.
* Close

************************************************************************
* MCI :: Save
******************
***  Function: Saves a file that's being recorded
***    Return: .T. or .F.
************************************************************************
PROCEDURE SAVE
LOCAL lcFilename

IF EMPTY(THIS.cFilename)
   lcFileName=PUTFILE('Save file to:', THIS.cFileName, THIS.cFileType)
   IF EMPTY(lcFilename)
      RETURN .F.
   ENDIF
   THIS.cFileName=lcFileName
ENDIF

IF THIS.MCIExec("Save "+THIS.cMCIObject+" "+THIS.cfilename)="*ERROR*"
   RETURN .F.
ENDIF

RETURN .T.
* Save

************************************************************************
* MCI :: Play
******************
***  Function: Plays the current MCI Device file
***      Pass: lcLocation  -   "START" or numeric location
***    Return: "*ERROR*" or ""
************************************************************************
PROCEDURE PLAY
LPARAMETER lvLocation
LOCAL lcLocation

lcLocation=""
IF TYPE("lvLocation")="C"
   IF  UPPER(lvLocation)="START"
      lcLocation="FROM 0"
   ELSE
      IF !EMPTY(lvLocation)
         lcLocation="FROM "+lvLocation
      ENDIF
   ENDIF
ENDIF

IF TYPE("lvLocation")="N"
   lcLocation="FROM "+ALLTRIM(STR(lvLocation))
ENDIF

RETURN THIS.MCIExec("Play "+THIS.cMCIObject+" "+lcLocation)
* Play

************************************************************************
* MCI :: Record
******************
***  Function: Records at a given position
***      Pass: lvLocation   -   Position to record to
***    Return: "*ERROR*" or ""
************************************************************************
PROCEDURE RECORD
LPARAMETERS lvLocation
LOCAL lcLocation

lcLocation=""
IF TYPE("lvLocation")="C"
   IF  UPPER(lvLocation)="START"
      lcLocation="FROM 0"
   ELSE
      IF !EMPTY(lvLocation)
         lcLocation="FROM "+lvLocation
      ENDIF
   ENDIF
ENDIF

IF TYPE("lvLocation")="N"
   lcLocation="FROM "+ALLTRIM(STR(lvLocation))
ENDIF

RETURN THIS.MCIExec("RECORD "+THIS.cMCIObject+" "+lcLocation+" overwrite")
* Record


************************************************************************
* MCI :: Pause
******************
***  Function: Pauses the currently playing file
***    Return: .T. or .F.
************************************************************************
PROCEDURE PAUSE
IF THIS.MCIExec("pause "+THIS.cMCIObject)="*ERROR*"
   RETURN .F.
ENDIF
RETURN .T.
* Pause


************************************************************************
* MCI :: Resume
******************
***  Function: Resumes a Paused device
***    Return: .T. or .F.
************************************************************************
PROCEDURE RESUME
IF THIS.MCIExec("Resume "+THIS.cMCIObject)="*ERROR*"
   RETURN .F.
ENDIF

RETURN .T.
* Resume

************************************************************************
* MCI :: Stop
******************
***  Function: Stops device.
***    Return: .T. or .F.
************************************************************************
PROCEDURE Stop
IF THIS.MCIExec("stop "+THIS.cMCIObject)="*ERROR*"
   RETURN .F.
ENDIF
RETURN .T.
* Stop

************************************************************************
* MCI :: SetTimeFormat
***********************
***  Function: Sets the time format used
***    Assume: Not all methods work with all devices
***      Pass: lcMethod     -  "MILLISECONDS","TRACK AND TIME","TIME"
***    Return: .T. or .F.
************************************************************************
PROCEDURE SetTimeFormat
LPARAMETERS lcMethod
LOCAL lcRetVal

lcMethod=IIF(TYPE("lcMethod")="C",UPPER(lcMethod),"MILLISECONDS")

DO CASE
   CASE lcMethod = "MILLISECONDS"
      lcRetVal=THIS.MCIExec("Set "+THIS.cMCIObject+;
         " time format milliseconds")
   CASE lcMethod = "TRACK AND TIME" OR lcMethod="TMSF"
      lcRetVal=THIS.MCIExec("Set "+THIS.cMCIObject+;
         " time format tmsf")
   CASE lcMethod = "TIME" OR lcMethod="MSF"
      lcRetVal=THIS.MCIExec("Set "+THIS.cMCIObject+;
         " time format msf")
   OTHERWISE
      lcRetVal=THIS.MCIExec("Set "+THIS.cMCIObject+;
         " time format milliseconds")
ENDCASE

IF lcRetVal="*ERROR*"
   RETURN .F.
ENDIF

RETURN .T.
* SetTimeFormat

************************************************************************
* MCI :: GetTimeFormat
***********************
***  Function: Returns a string of the current time format.
***            Base format is MS or MILLISECONDS. Others are FRAME,
***            SAMPLES, TIME etc.
***    Return: Time Format string
************************************************************************
PROCEDURE GetTimeFormat
LOCAL lcRetVal

RETURN UPPER(THIS.MCIExec("Status "+THIS.cMCIObject+" time format"))
* GetTimeFormat

************************************************************************
* MCI :: GetTrack
******************
***  Function: Returns the current track
***    Assume: Time Format is "MILLISECONDS"
***    Return: track number or -1
************************************************************************
PROCEDURE GetTrack
LOCAL lcRetVal

lcRetVal=THIS.MCIExec("Status "+THIS.cMCIObject+" current track")
IF lcRetVal="*ERROR*"
   RETURN -1
ENDIF

RETURN VAL(lcRetVal)
* GetTrack


************************************************************************
* MCI :: GetTrackLength
************************
***  Function: Returns the length of the current or specified track
***      Pass: lnTrack  -  nothing current track or 0
***                        or track number
***    Return: -1
************************************************************************
PROCEDURE GetTrackLength
LPARAMETER lnTrack
LOCAL lcRetVal,llNumeric

*** Determine time format - String or Numeric (Milliseconds)
IF UPPER(THIS.MCICommand("Status","time format")) # "MILLISECONDS"
   llNumeric=.F.
ELSE
   llNumeric=.T.
ENDIF

lnTrack=IIF(TYPE("lnTrack")="N",lnTrack,0)

IF lnTrack=0
   lnTrack=THIS.GetTrack()
   IF lnTrack#-1
      lcRetval=THIS.MCIExec("Status "+THIS.cMCIObject+" Length Track "+STR(lnTrack))
   ELSE
      lcRetVal="*ERROR*"
   ENDIF
ELSE
   lcRetval=THIS.MCIExec("Status "+THIS.cMCIObject+" Length track "+STR(lnTrack))
ENDIF

IF lcRetVal="*ERROR*"
   IF llNumeric
      RETURN -1
   ELSE
      RETURN lcRetval
   ENDIF
ENDIF

IF llNumeric
   RETURN VAL(lcRetVal)
ENDIF

RETURN lcRetVal
* GetTrackLength

************************************************************************
* MCI :: GetLength
************************
***  Function: Returns the length of the total device (CD VideoDisk)
***      Pass: lnTrack  -  nothing current track or 0
***                        or track number
***    Assume: Time Format is previously set time format
***    Return: -1
************************************************************************
PROCEDURE GetLength
LOCAL lcRetVal,llNumeric

*** Determine time format - String or Numeric (Milliseconds)
IF UPPER(THIS.MCICommand("Status","time format")) # "MILLISECONDS"
   llNumeric=.F.
ELSE
   llNumeric=.T.
ENDIF

lcRetval=THIS.MCIExec("Status "+THIS.cMCIObject+" Length")

IF lcRetVal="*ERROR*"
   IF llNumeric
      RETURN -1
   ELSE
      RETURN lcRetVal
   ENDIF
ENDIF

IF llNumeric
   RETURN VAL(lcRetVal)
ENDIF

RETURN lcRetVal
* GetLength


************************************************************************
* MCI :: GetTracks
******************
***  Function: Returns the number of tracks
***    Return: No of tracks or -1 on error
************************************************************************
PROCEDURE GetTracks
lcRetVal=THIS.MCIExec("Status "+THIS.cMCIObject+" number of tracks")

IF lcRetVal="*ERROR*"
   RETURN -1
ENDIF

RETURN VAL(lcRetVal)
* GetTracks

************************************************************************
* MCI :: GetTrackPosition
***************************
***  Function: Returns the current numeric position of the track in
***            milliseonds
***    Assume: Time Format is previously set time format
***    Return: numeric position or 0 if an error occurred
***            or Time String if time format is not MILLISECONDS
************************************************************************
PROCEDURE GetTrackPosition
LOCAL lcRetVal,lnTrack,llNumeric

lnTrack=THIS.GetTrack()

*** Determine time format - String or Numeric (Milliseconds)
IF UPPER(THIS.MCICommand("Status","time format")) # "MILLISECONDS"
   llNumeric=.F.
ELSE
   llNumeric=.T.
ENDIF

IF lnTrack#-1
   *** Start Position of track (of total)
   lcRetVal=THIS.MCIExec("status "+THIS.cMCIObject+;
      " position track "+STR(lnTrack,3,0))

   *** Current Position (of total)
   lcRetVal2=THIS.MCIExec("status "+THIS.cMCIObject+;
      " position")
ELSE
   lcRetVal="*ERROR*"
ENDIF

IF lcRetVal="*ERROR*" .OR. lcRetVal2="*ERROR*"
   IF llNumeric
      RETURN -1
   ELSE
      RETURN "*ERROR*"
   ENDIF
ENDIF

IF llNumeric
   RETURN VAL(lcRetVal2)-VAL(lcRetVal)
ENDIF

RETURN lcRetval2

ENDPROC
* GetTrackPosition

************************************************************************
* MCI :: SetTrackPosition
**************************
***  Function: Goes to a given track position
***      Pass: lvPosition  -   milliseconds format  (Numeric)
***                            or Track:Minutes:Seconds:Frame (char)
***    Assume: Takes some time! Use PLAY(nTrackPosition) instead
***            Time Format is set to properly support value passed
***    Return: .T. or .F.
************************************************************************
PROCEDURE SetTrackPosition
LPARAMETERS lvPosition

DO CASE
   CASE TYPE("lvPosition")="C"
      lcRetVal=THIS.MCIExec("SEEK "+THIS.cMCIObject+" TO "+;
         lvPosition)
   CASE TYPE("lvPosition")="N"
      lcRetVal=THIS.MCIExec("SEEK "+THIS.cMCIObject+" TO "+;
         STR(lvPosition) )
   OTHERWISE
      lcRetVal="*ERROR*"
ENDCASE

IF lcRetVal="*ERROR*"
   RETURN .F.
ENDIF

RETURN .T.
* SetTrackPosition

************************************************************************
* MCI :: GetSize
******************
***  Function: Visual Location of Window for Video
***      Pass:  Window Coordinate Vars returned by reference!!!
***    Return: .T. or .F.
************************************************************************
PROCEDURE GetSize
LPARAMETER lnWidth,lnHeight
LOCAL lnSpace2,lnSpace3,lcRetStr

lcRetStr=THIS.MCIExec('Where '+THIS.cMCIObject+' Window')

lnSpace2=AT(' ',lcRetStr,2)
lnSpace3=AT(' ',lcRetStr,3)

lnWidth=VAL(SUBSTR(lcRetStr,lnSpace2+1,lnSpace3-lnSpace2-1))
lnHeight=VAL(RIGHT(lcRetStr,LEN(lcRetStr)-lnSpace3))
ENDPROC
* Where

************************************************************************
* MCI :: SetSize
******************
***  Function: Sets size and dimensions of window for visual devices
***      Pass: Window Coordinates
***    Return: .T. or .F.
************************************************************************
PROCEDURE SetSize
LPARAMETER lnLeft,lnTop,lnWidth,lnHeight

lnLeft=IIF(TYPE("lnLeft")="N",lnLeft,0)
lnTop=IIF(TYPE("lnTop")="N",lnTop,0)

*** Reset to default size
lnWidth=IIF(TYPE("lnWidth")="N",lnWidth,THIS.nWidth)
lnHeight=IIF(TYPE("lnHeight")="N",lnHeight,THIS.nHeight)

IF THIS.MCIExec("Put "+THIS.cMCIObject+" window at "+;
      ALLTRIM(STR(lnLeft))+" "+;
      ALLTRIM(STR(lnTop))+" "+;
      ALLTRIM(STR(lnWidth))+" "+;
      ALLTRIM(STR(lnHeight)) )="*ERROR*"
   RETURN .F.
ELSE
   RETURN .T.
ENDIF
ENDPROC


************************************************************************
* MCI :: GetMode
******************
***  Function: Returns current state of device
***    Assume:
***    Return: Mode String ("playing","stopped","paused","seeking")
************************************************************************
PROCEDURE GetMode
RETURN UPPER(THIS.MCIExec("Status "+THIS.cMCIObject+" Mode"))
* GetMode


************************************************************************
* MCI :: MSToTime
******************
***  Modified: 04/23/95
***  Function: Converts seconds to a Time String
***
***      Pass: pnSecs  -  Seconds to convert
***
***    Return: TIME() type string
************************************************************************
PROCEDURE MSToTime
LPARAMETERS pnSecs
LOCAL lnHours,lnMins,lcSecStr

pnSecs=IIF(TYPE("pnSecs")="N",pnSecs/1000,0)

lnhours=INT(pnSecs/3600)
pnSecs=pnsecs-lnHours*3600
lnMins=INT(pnSecs/60)
pnSecs=pnSecs-lnMins*60

lcSecStr=STR(lnHours,2)+":"+STR(lnMins,2)+":"+STR(pnSecs,2)
lcSecStr=STRTRAN(lcSecStr," ","0")

RETURN lcSecStr
*EOP MSToTime

************************************************************************
* MCI :: TimeToMS
******************
***  Procedure to convert a time string to seconds (numeric)
***
***     pass:   TIMESTR    -  Time String in 24 hour format
***		                    FORMAT "15:15:15"  (Hours/mins/secs)
***
***  returns:   number of seconds since midnight
***             -1  -  bad timestring passed
**************************************************************************
PROCEDURE TimeToMS
PARAMETER timestr

PRIVATE timestr,t_seconds,HOURS,minutes,SECONDS

HOURS=SUBSTR(timestr,1,2)
minutes=SUBSTR(timestr,4,2)

IF EMPTY(HOURS) .OR. EMPTY(minutes)
   RETURN -1
ENDIF

HOURS=VAL(HOURS)
minutes=VAL(minutes)
*** just convert seconds
SECONDS=VAL(SUBSTR(timestr,7,2))

IF HOURS>23 OR minutes>59 OR SECONDS>59 OR ;
      HOURS<0 OR minutes<0 OR SECONDS<0
   RETURN -1
ENDIF

t_seconds=INT(HOURS*3600+minutes*60+SECONDS)
RETURN t_seconds*1000
* TimeToMS


************************************************************************
* MCI :: MCIExec
******************
***  Modified: 06/15/95
***  Function: MCIExecute API function
***      Pass: lcMCIStr  -   MCI Command String
***    Return: Result String if applicable ("" if none)
***            "*ERROR*"+ error number if an error occurs
************************************************************************
PROCEDURE MCIExec
LPARAMETERS lcMCIStr
LOCAL lfMCIExec,lcRetStr,lnRetVal

*** mciSendString DECLARED in INIT
lcRetStr=SPACE(80)
lnRetval=mciSendString(lcMCIStr,@lcRetStr,LEN(lcRetStr),0)

IF lnRetVal>0
   RETURN "*ERROR*"+STR(lnRetVal)
ENDIF

RETURN TRIM(STRTRAN(lcRetStr,CHR(0),""))
*EOP MCIExecute


************************************************************************
* MCI :: MCICommand
********************
***  Modified: 06/15/95
***  Function: Like MCIExec except it doesn't require knowledge of the
***            object name alias.
***      Pass: lcMCIVerb    -   MCI Command Verb
***            lcMCIOptions -   Optional parameters following MCI ID
***    Return: Result String if applicable ("" if none)
***            "*ERROR*"+ error number if an error occurs
************************************************************************
PROCEDURE MCICommand
LPARAMETERS lcMCIVerb,lcMCIOptions
LOCAL lcRetStr,lnRetval

lcMCIOptions=IIF(TYPE("lcMCIOptions")="C",lcMCIOptions,"")

*** mciSendString DECLARED in INIT
lcRetStr=SPACE(80)
lnRetVal=mciSendString(lcMCIVerb+" "+THIS.cMCIObject+" "+lcMCIOptions,;
   @lcRetStr,LEN(lcRetStr),0)

IF lnRetVal>0
   RETURN "*ERROR*"+STR(lnRetVal)
ENDIF

RETURN TRIM(STRTRAN(lcRetStr,CHR(0),""))
*EOP MCIExecute

************************************************************************
* MCI :: MCIErrorString
************************
***  Function: Returns an error string that explains a given
***            MCI Error number. Pass either the error number
***            as a number, or pass an "*ERROR*xxxx" string
***            returned by many of the above functions.
***      Pass: lvError   -   Error Number (Numeric) or
***                          "*ERROR*xxxx"  String
***    Return: Error String or ""
************************************************************************
PROCEDURE MCIErrorString
LPARAMETERS lvError
LOCAL lcErrorString,lnError

***  BOOL mciGetErrorString(fdwError, lpszErrorText, cchErrorText)
***
***  DWORD fdwError;	/* error code	*/
***  LPTSTR lpszErrorText;	/* address of buffer for error string	*/
***  UINT cchErrorText;	/* size of buffer, in characters	*/
DECLARE INTEGER mciGetErrorString ;
   IN WINMM.DLL ;
   INTEGER nErrorno, ;
   STRING @cBuffer, ;
   INTEGER nBufSize

*** This is my normal comment mode
lnError=0
IF TYPE("lvError")="C"
   IF LEFT(lvError,7)="*ERROR*"
      lnError=VAL(SUBSTR(lvError,8))
   ENDIF
ENDIF

IF TYPE("lvError")="N"
   lnError=lvError
ENDIF

lcErrorString=SPACE(256)
=mciGetErrorString(lnError,@lcErrorString,LEN(lcErrorString))

RETURN TRIM(CHRTRAN(lcErrorString,CHR(0),""))
* MCIErrorStr


ENDDEFINE
*** End of Class - MCI


*************************************************************
DEFINE CLASS mciVideo AS MCI
*************************************************************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1995
***   Contact: (503) 386-2087  / 76427,2363@compuserve.com
***  Modified: 06/19/95
***
***  Function: Video Specific Version Of The Mci Class
*************************************************************


************************************************************************
* mciVideo :: Init
******************
***      Pass: lcFilename    -  File name to play
***            lcWindowTitle -  The Window that will hold video
***                             Don't pass for device window
************************************************************************
PROCEDURE Init
LPARAMETERS lcFileName,lcWindowTitle

THIS.cFileType="AVI"

*** Call the old Init routine
=EVALUATE(THIS.parentclass+"::init()")

IF !EMPTY(lcFileName)
   THIS.open(lcFileName,lcWindowTitle)
ENDIF

ENDPROC
* Init


************************************************************************
* mciVideo :: Open
******************
***  Function: Opens a file and initializes the MCI device for it.
***            Video Version overrides for window type and AVI extension
***            on the file open dialog
***      Pass: lcFilename    -   File to open
***            lcWindowTitle -   Window title that is to receive video
***                              or don't pass for seperate window
***    Return: .T. or .F.
************************************************************************
PROCEDURE OPEN
LPARAMETERS lcFileName,lcWindowTitle,p3
LOCAL lcOptions,lhThisWin

lcWindowTitle=IIF(TYPE("lcWindowTitle")="C",lcWindowTitle,"")

*** Need FoxTools for VFP Window Handle
IF !"FOXTOOLS" $ SET("LIBRARY")
   SET LIBRARY TO HOME()+"FOXTOOLS" ADDITIVE
ENDIF

IF EMPTY(lcWindowTitle)
   *** Use standalone window that's parent to _SCREEN
   lcOptions=" Parent "+ALLTRIM(STR(MainHwnd()))
ELSE
   *** Pop video into VFP Form
   lhThisWin=_WhToHwnd(_WFindTitl(lcWindowTitle))
   lcOptions=" Style Child Parent "+ALLTRIM(STR(lhThisWin))
ENDIF

*** Call Base class code with AVI option
RETURN EVALUATE( THIS.parentclass+"::open(lcFileName,'AVI',lcOptions)" )
* Open

ENDDEFINE
*EOC mciVideo



*************************************************************
DEFINE CLASS MCIWave AS MCI
*************************************************************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1995
***   Contact: (503) 386-2087  / 76427,2363@compuserve.com
***  Modified: 06/27/95
***  Function: Wave file specific MCI class
***            Only supports playing at this time
*************************************************************

************************************************************************
* mciWave ::  Init
******************
***      Pass: lcFilename    -  File name to play
***            lcMethod      -  "PLAY"*, "RECORD"
************************************************************************
FUNCTION INIT
LPARAMETERS lcFileName,lcMethod

lcMethod=IIF(TYPE("lcMethod")="C",lcMethod,"")

THIS.cFileType="WAV"

*** Call the old Init routine
=EVALUATE(THIS.parentclass+"::init()")


IF !EMPTY(lcFileName)
   THIS.open(lcFileName,lcMethod)
ENDIF

ENDPROC
* Init

************************************************************************
* mciWave ::  Open
******************
***  Function: Opens a file and initializes the MCI device for it.
***            Video Version overrides for window type and AVI extension
***            on the file open dialog
***      Pass: lcFilename    -   File to open
***    Return: .T. or .F.
************************************************************************
PROCEDURE OPEN
LPARAMETERS lcFileName

*** Call Base class code with AVI option
RETURN EVALUATE( THIS.parentclass+"::open(lcFileName,'WAV')" )
* Open

************************************************************************
* mciWave ::  OpenForRecording
******************************
***  Function: Opens Wave device for recording.
***      Pass: lcFileName   -  File to record to
***            lnBits       -  8*, 16
***            lnChannel    -  1*, 2 etc.  mono stereo
***            lcSampleRate -  "11k", "22k"
***    Assume: Wave Audio recording is very limited. Only 9 seconds
***            of audio can be recorded and the default driver
***            MCIWAVE.DRV only supports 8 bit sound at 11khz samples
***            in Mono.
***      Todo: Add parameters for sample rate, channels and bits
***    Return: Error string or ""
************************************************************************
PROCEDURE OpenForRecording
LPARAMETERS lcFileName
LOCAL lcOptions

*** Wave Audio recording

RETURN EVALUATE( THIS.parentclass+;
   "::OpenForRecording(lcFileName,'WAVEAUDIO','buffer 9')" )
* OpenForRecording


************************************************************************
* mciWave :: PlayWav
********************
***  Function: This function is not directly related to MCI
***            but it allows easy non-synchronous playback of a WAV
***            file when no movement control is required.
***
***            Plays the Wave File or WIN.INI
***            [Sounds] Entry specified in the
***            parameter. If the .WAV file or
***            System Sound can't be found,
***            SystemDefault beep is played
***    Assume: Runs only under Windows
***            uses MMSYSTEM.DLL  (Win 3.1)
***                 WINMM.DLL  (32 bit Win)
***      Pass: pnPlayType - 1*- Ascynch, 2 - Nodefault sound 0 -Synch
***                         4 - Kill currently playing sound
***                         8 - Continous
***                         Can be combined by adding together
***
***    Return: .t. if Wave was played .f. otherwise
*************************************************************************
PROCEDURE Playwav
LPARAMETER pnPlayType
LOCAL lhPlaySnd,llRetVal

pnPlayType=IIF(TYPE("pnPlayType")="N",pnPlayType,1)

llRetVal=.F.

DECLARE INTEGER PlaySound ;
   IN WINMM.dll  ;
   STRING cWave, INTEGER nModule, INTEGER nType

IF PlaySound(THIS.cFileName,0,pnPlayType)=1
   llRetVal=.T.
ENDIF

RETURN llRetVal
*EOP PLAYWAV

ENDDEFINE
*EOC MCIWave


*************************************************************
DEFINE CLASS MCICd AS MCI
*************************************************************
***    Author: Rick Strahl
***            (c) West Wind Technologies, 1995
***   Contact: (503) 386-2087  / 76427,2363@compuserve.com
***  Modified: 06/27/95
***    Assume: Time Format used: Track,Minutes,Seconds,Frames
***                     Example: 01:02:15:00
***  Function: Plays Audio CD's
*************************************************************

************************************************************************
* mciCD ::  Open
******************
***  Function: Opens the CD device
***    Return: .T. .or. .F.
************************************************************************
PROCEDURE OPEN

IF THIS.mciExec("open  cdAudio ALIAS "+THIS.cMCIObject+" shareable")#"*ERROR*"
   THIS.SetTimeFormat()
   RETURN .T.
ENDIF
RETURN .F.
* Open


************************************************************************
* mciCD ::  Close
******************
***  Function: Overrides MCI close method by forcing cd to stop!
***    Return: .T. or .F.
************************************************************************
PROCEDURE CLOSE
THIS.stop
RETURN EVALUATE( THIS.parentclass+"::close()" )
* Destroy

************************************************************************
* mciCD ::  Play
******************
***  Function:  Override parentclass to allow using track/time format
***      Pass:  lcPosition   -   "01:02:15"  Track 1, 2 mins 15 secs
***                              or use Milliseconds
***    Return:  .t. or .f.
************************************************************************
PROCEDURE PLAY
LPARAMETERS lcPosition
LOCAL lcOldFormat,llChangeFormat,lcRetVal

DO CASE
   CASE TYPE("lcPosition")="C"
      lcOldFormat=THIS.GetTimeFormat()
      llChangeFormat=.F.

      IF lcOldFormat#"TMSF"
         THIS.SetTimeFormat("TRACK AND TIME")
         llChangeFormat=.T.
      ENDIF

      lcRetVal=THIS.MCIEXEC("play "+THIS.cMCIObject+" FROM "+lcPosition)

      IF llChangeFormat
         THIS.SetTimeFormat(lcOldFormat)
      ENDIF

   CASE TYPE("lcPosition")="N"
      lcRetVal=THIS.MCIEXEC("play "+THIS.cMCIObject+" FROM "+lcPosition)

   OTHERWISE
      RETURN EVALUATE(THIS.parentclass+"::play()")
ENDCASE

IF lcRetVal="*ERROR*"
   RETURN .F.
ENDIF

RETURN .T.
* Play

************************************************************************
* MCICd :: Resume
*********************************
***  Function: Resume doesn't appear to work for CD's, so use PLay
***            instead.
***    Return: .T. or .F.
************************************************************************
FUNCTION RESUME
RETURN THIS.Play()
* Resume

************************************************************************
* MCICd :: MoveTrack
*********************************
***  Function: Moves a number of tracks forward or backward.
***      Pass: lnMovement - Number of tracks to move
***                         negative number to move back
***    Return: .T. or .F.
************************************************************************
FUNCTION MoveTrack
LPARAMETER lnMovement
LOCAL lnCurTrack, llRetVal

lnCurTrack=THIS.GetTrack()

lcOldFormat=THIS.GetTimeFormat()
llChangeFormat=.F.

IF lcOldFormat#"TMSF"
   THIS.SetTimeFormat("TRACK AND TIME")
   llChangeFormat=.T.
ENDIF

llRetVal=THIS.Play(STR(lnCurTrack+lnMovement,2)+":00:00")

IF llChangeFormat
   THIS.SetTimeFormat(lcOldFormat)
ENDIF

RETURN llRetVal
* MoveTrack

ENDDEFINE
* EOC MCICd
